@@ -3,7 +3,7 @@ package App::Maisha::Plugin::Base;
 use strict;
 use warnings;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 #----------------------------------------------------------------------------
 # Public API
@@ -12,9 +12,9 @@ sub new {
     my $class = shift;
     my $self = {
         source      => 'maisha',
-        useragent   => 'Maisha/0.01 (Perl)',
+        useragent   => 'Maisha/0.16 (Perl)',
         clientname  => 'Maisha',
-        clientver   => '0.01',
+        clientver   => '0.16',
         clienturl   => 'http://maisha.grango.org'
     };
 
@@ -126,13 +126,14 @@ L<App::Maisha>
 
 =head1 AUTHOR
 
-  Copyright (c) 2009-2010 Barbie <barbie@cpan.org> for Grango.org.
+  Barbie, <barbie@cpan.org>
+  for Miss Barbell Productions <http://www.missbarbell.co.uk>.
 
-=head1 LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-  This program is free software; you can redistribute it and/or modify it
-  under the same terms as Perl itself.
+  Copyright (C) 2009-2012 by Barbie
 
-  See http://www.perl.com/perl/misc/Artistic.html
+  This module is free software; you can redistribute it and/or
+  modify it under the Artistic License v2.
 
 =cut
