@@ -3,7 +3,7 @@ package App::Maisha::Plugin::Twitter;
 use strict;
 use warnings;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 #----------------------------------------------------------------------------
 # Library Modules
@@ -46,16 +46,19 @@ sub new {
 
 sub login {
     my ($self,$config) = @_;
+    my $api;
 
     unless($config->{username}) { warn "No username supplied\n"; return }
 
-    my $api = Net::Twitter->new(
-#        traits              => [qw/API::REST OAuth WrapError/],
-        traits              => [qw/API::REST OAuth/],
-        consumer_key        => $self->{consumer_key},
-        consumer_secret     => $self->{consumer_secret},
-        ssl                 => 1
-    );
+    eval {
+        $api = Net::Twitter->new(
+#            traits              => [qw/API::REST OAuth WrapError/],
+            traits              => [qw/API::REST OAuth/],
+            consumer_key        => $self->{consumer_key},
+            consumer_secret     => $self->{consumer_secret},
+            ssl                 => 1
+        );
+    };
 
     unless($api) {
         warn "Unable to establish connection to Twitter API\n";
@@ -64,28 +67,34 @@ sub login {
 
     # for testing purposes we don't want to login
     if(!$config->{test}) {
-        my $datafile = $config->{home} . '/.maisha/twitter.dat';
-        my $access_tokens = eval { retrieve($datafile) } || {};
+        eval {
+            my $datafile = $config->{home} . '/.maisha/twitter.dat';
+            my $access_tokens = eval { retrieve($datafile) } || {};
 
-        if ( $access_tokens && $access_tokens->{$config->{username}}) {
-            $api->access_token($access_tokens->{$config->{username}}->[0]);
-            $api->access_token_secret($access_tokens->{$config->{username}}->[1]);
-        }
-        else {
-            my $auth_url = $api->get_authorization_url;
-            print " Authorize this application at: $auth_url\nThen, enter the PIN# provided to continue: ";
+            if ( $access_tokens && $access_tokens->{$config->{username}}) {
+                $api->access_token($access_tokens->{$config->{username}}->[0]);
+                $api->access_token_secret($access_tokens->{$config->{username}}->[1]);
+            } else {
+                my $auth_url = $api->get_authorization_url;
+                print " Authorize this application at: $auth_url\nThen, enter the PIN# provided to continue: ";
 
-            my $pin = <STDIN>; # wait for input
-            chomp $pin;
+                my $pin = <STDIN>; # wait for input
+                chomp $pin;
 
-            # request_access_token stores the tokens in $nt AND returns them
-            my @access_tokens = $api->request_access_token(verifier => $pin);
-            $access_tokens->{$config->{username}} = \@access_tokens;
+                # request_access_token stores the tokens in $nt AND returns them
+                my @access_tokens = $api->request_access_token(verifier => $pin);
+                $access_tokens->{$config->{username}} = \@access_tokens;
 
-            mkpath( $config->{home} . '/.maisha' );
+                mkpath( $config->{home} . '/.maisha' );
 
-            # save the access tokens
-            store $access_tokens, $datafile;
+                # save the access tokens
+                store $access_tokens, $datafile;
+            }
+        };
+
+        if($@) {
+            warn "Unable to login to Twitter\n";
+            return 0;
         }
     }
 
@@ -103,10 +112,12 @@ sub _build_users {
     my $self = shift;
     my %users;
 
-    my $f = $self->api->friends();
-    if($f && @$f)   { for(@$f) { next unless($_); $users{$_->{screen_name}} = 1 } }
-    $f = $self->api->followers();
-    if($f && @$f)   { for(@$f) { next unless($_); $users{$_->{screen_name}} = 1 } }
+    eval {
+        my $f = $self->api->friends();
+        if($f && @$f)   { for(@$f) { next unless($_); $users{$_->{screen_name}} = 1 } }
+        $f = $self->api->followers();
+        if($f && @$f)   { for(@$f) { next unless($_); $users{$_->{screen_name}} = 1 } }
+    };
 
     $self->users(\%users);
 }
@@ -128,12 +139,12 @@ sub api_user_timeline {
 
 sub api_friends {
     my $self = shift;
-    $self->api->friends();
+    $self->api->following();
 }
 
 sub api_friends_timeline {
     my $self = shift;
-    $self->api->friends_timeline();
+    $self->api->following_timeline();
 }
 
 sub api_public_timeline {
@@ -144,6 +155,9 @@ sub api_public_timeline {
 sub api_followers {
     my $self = shift;
     $self->api->followers();
+
+    # below is meant to be the same as the above, but it isn't :(
+    #$self->api->lookup_users( { user_id => $self->api->followers_ids() } );
 }
 
 sub api_replies {
@@ -290,13 +304,14 @@ L<Net::Twitter>
 
 =head1 AUTHOR
 
-  Copyright (c) 2009-2010 Barbie <barbie@cpan.org> for Grango.org.
+  Barbie, <barbie@cpan.org>
+  for Miss Barbell Productions <http://www.missbarbell.co.uk>.
 
-=head1 LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-  This program is free software; you can redistribute it and/or modify it
-  under the same terms as Perl itself.
+  Copyright (C) 2009-2012 by Barbie
 
-  See http://www.perl.com/perl/misc/Artistic.html
+  This module is free software; you can redistribute it and/or
+  modify it under the Artistic License v2.
 
 =cut
